/****************************************************
  File:       accept_birth.c
  Written by: Jaya Satagopan (satago@biost.mskcc.org)
  Date:       September 23, 1997

  Purpose:

    Function ACCEPT_BIRTH computes the acceptance
    probability for the birth of a new QTL.
****************************************************/

#include "revjump.h"

extern double get_log_model_ratio(int, int, double *,
	double *, double, double *, double, int *, int *);
extern double get_qtl_ratio(int, int, int, int *, int *, int *,
	double *, double *, double *, double, double *);
extern double get_position_ratio(int, int, double *);
extern double get_par_ratio(int, double, double *, double *,
	double, double, double, double);
extern double get_qtl_number_ratio(double, int);
extern double get_q_birth(int, int, int, int *, int *, double,
	double *, double *, double *, double, double *);
extern double get_q_death(int);
extern double get_jacobian(double, double);

extern float  genunf(float, float);
extern float  genunf(float, float);
extern double dist_rec(double, int, double *);
extern void   f1_prob(int, int, double, double, double *);
extern int    get_new_interval(int, int, double *);
extern int    *get_new_qtl_genotype(int, int *, double *, double *, double);
extern double get_new_locus(int);
extern void   get_birth_effect(long int, long int, int *, int *, 
			     double *, double *, double *, double);

double accept_birth(int ntrait, int nmark, int nqtl, double *y,
		   double mu, double *effect, double sigmasq,
		   double *modified_parms, int qtl_genotype[][MAXQTL],
		   int *new_qtl_genotype, int marker_genotype[][MAXMARKER],
		   double *dist, double *cumulative_dist, 
		   double *recombination, double *lambda,
		   double new_position, double mu_prior_mn, 
		   double mu_prior_sd, double effect_prior_mn,
		   double effect_prior_sd, double sig_prior_a1, 
		   double sig_prior_a2, double qtl_prior_mn,
		   double effect_uniform_number, double V_val,
		   double *pival){

  double log_model_ratio, log_qtl_geno_ratio, log_par_ratio, 
	 log_position_ratio, log_qtl_number_ratio, 
	 log_q_birth, log_q_death, log_jacobian, log_accept_birth_prob;
  double prob_birth, prob_death;
  int i,j;
/*
  double *work_cum_dist = (double *) malloc((nmark-1)*sizeof(double));

  for(i=0;i<(nmark-1);i++)
    work_cum_dist[i] = cumulative_dist[i]/100.0;
*/

  /* COMPUTE FULL CONDITIONAL RATIO FOR BIRTH PROCESS */

  log_model_ratio = get_log_model_ratio(ntrait, nqtl, &y[0], &modified_parms[0],
		      mu, &effect[0], sigmasq, 
		      &qtl_genotype[0][0], &new_qtl_genotype[0]);


  log_qtl_geno_ratio = get_qtl_ratio(ntrait, nmark, nqtl,
			 &qtl_genotype[0][0], &new_qtl_genotype[0], 
			 &marker_genotype[0][0], &dist[0], &recombination[0],
			 &lambda[0], new_position,
			 &pival[0]);

  log_position_ratio = get_position_ratio(nqtl, nmark, &cumulative_dist[0]);


  log_par_ratio = get_par_ratio(nqtl, mu, &effect[0], &modified_parms[0],
		  mu_prior_mn, mu_prior_sd*mu_prior_sd, effect_prior_mn, 
		  effect_prior_sd*effect_prior_sd);


  log_qtl_number_ratio = get_qtl_number_ratio( qtl_prior_mn, nqtl);


  prob_birth = qtl_prior_mn / (nqtl+1);
  prob_death = 1/prob_birth;

  if( prob_birth > 1 ) prob_birth = 1.0;
  if( prob_death > 1 ) prob_death = 1.0;
  prob_birth = 0.4 * prob_birth;
  prob_death = 0.4 * prob_death;

/*
printf("\n");
printf("prob birth is %f\n", prob_birth);
printf("prob death is %f\n", prob_death);
printf("\n");
*/

  log_q_birth = get_q_birth(ntrait, nmark, nqtl, &new_qtl_genotype[0],
			&marker_genotype[0][0], new_position, &dist[0],
			&cumulative_dist[0], &recombination[0],
			effect_uniform_number, &pival[0]);

  log_q_death = get_q_death(nqtl);

  log_jacobian = get_jacobian(sigmasq, V_val);

  log_accept_birth_prob = log_model_ratio + log_qtl_geno_ratio +
		          log_position_ratio + log_par_ratio +
		          log_qtl_number_ratio +
		          log(prob_death) - log(prob_birth) +
		          log_q_death - log_q_birth + log_jacobian;

/*
printf("\n");
printf("model ratio is %f\n", log_model_ratio);
printf("qtl geno ratio is %f\n", log_qtl_geno_ratio);
printf("position ratio is %f\n", log_position_ratio);
printf("parms ratio is %f\n", log_par_ratio);
printf("qtl number ratio is %f\n", log_qtl_number_ratio);
printf("prob death is %f\n", log(prob_death));
printf("prob birth is %f\n", log(prob_birth));
printf("q death is %f\n", log_q_death);
printf("q birth is %f\n", log_q_birth);
printf("jacobian is %f\n", log_jacobian);
printf("log accept prob is %f\n", log_accept_birth_prob);
printf("\n");
*/

  return log_accept_birth_prob;
}
