/*****************************************************
  File:        accept_prob.c
  Date:        September 8, 1997
  Written by:  Jaya Satagopan (satago@biost.mskcc.org)

  Purpose:
    Calculate the metropolis-hastings acceptance
    probability for lambda versus new_lambda.
    Return whichever value of lambda is accepted.
*****************************************************/

#include "revjump.h"

extern float genunf(float,float);
extern void which_marker(int, int, int, int *, int *, int, double *,
			 double *, int, int, int *); 

/*********************************
   Function GET_ACCEPT_PROB
*********************************/

void get_accept_prob(int ntrait, int nmark, int nqtl, int iqtl,
                  int marker_genotype[][MAXMARKER],
                  int qtl_genotype[][MAXQTL],
                  int dist_old, int dist_new,
                  int flank_left, int flank_right,
                  double *dist, double r1_old, double R_old,
                  double r1_new, double R_new,
                  double *lambda, double *new_lambda, 
		  double *accept_prob){

  double tolerance = 0.000001, p1, p2;
  int new_mark1, new_mark2, old_mark1, old_mark2, i, j;
  double *pival = (double *) malloc(2*sizeof(double));
  double *qival = (double *) malloc(2*sizeof(double));
  int *markers = (int *) malloc(2*sizeof(int));

  if( dist[dist_old] > tolerance )
    accept_prob[0] = dist[dist_new] / dist[dist_old];
  else accept_prob[0] = 1.0;

  for(j=0;j<ntrait;j++){
    if( nqtl == 1 ){
      new_mark1 = marker_genotype[j][dist_new];
      new_mark2 = marker_genotype[j][dist_new+1];
    }
    else{
      which_marker(iqtl, j, nqtl, &marker_genotype[0][0],
                             &qtl_genotype[0][0], dist_new,
                             &lambda[0], &new_lambda[0], flank_left,
                             flank_right, &markers[0]);
      new_mark1 = markers[0];
      new_mark2 = markers[1];
    }
    old_mark1 = marker_genotype[j][dist_old];
    old_mark2 = marker_genotype[j][dist_old+1];

    f1_prob( new_mark1, new_mark2, r1_new, R_new, &pival[0]);
    f1_prob( old_mark1, old_mark2, r1_old, R_old, &qival[0]);
    if( qtl_genotype[j][iqtl] == 1 ){
      p1 = pival[0];
      p2 = qival[0];
    }
    else{
      p1 = pival[1];
      p2 = qival[1];
    }

    if( (p2 > tolerance) && (p1 > tolerance) )
      accept_prob[0] = accept_prob[0] * p1/p2;
    else{
      accept_prob[0] = 0.0;
      break;
    }
  }
}
