/****************************************************
  File:       birth.c
  Written by: Jaya Satagopan (satago@biost.mskcc.org)
  Date:       September 4, 1997

  Purpose:

    Function BIRTH performs the "birth step" for
    adding a new QTL to the model.
****************************************************/

#include "revjump.h"

extern float  genunf(float, float);
extern float  genunf(float, float);
extern double dist_rec(double, int, double *);
extern void   f1_prob(int, int, double, double, double *);
extern int    get_new_interval(int, int, double *);
extern int    *get_new_qtl_genotype(int, int *, double *, double *, double,
				    int *, double *);
extern double get_new_locus(int);
extern void   get_birth_effect(long int, long int, int *, int *, 
			     double *, double *, double *, double,
			     double *, double *, double *, double *,
			     double *, double *, double *, double *,
			     double *);
extern double accept_birth(int, int, int, double *, double, double *, double,
	double *, int *, int *, int *, double *, double *, double *,
	double *, double, double, double, double, double, double, double, 
	double, double, double, double *);
extern void setvals(int, int *, double *, double *, double *, double *,
	double, int *, int *);

void birth( int ntrait, int nmark, int *nqtl, double *y,
	    double *mu, double *effect, double sigmasq,
	    int qtl_genotype[][MAXQTL], int marker_genotype[][MAXMARKER],
	    double *dist, double *cumdist, double *recombination,
	    double *lambda, double mu_prior_mn, double mu_prior_sd,
	    double effect_prior_mn, double effect_prior_sd,
	    double sig_prior_a1, double sig_prior_a2, double qtl_prior_mn,
	    int *nmove,
	    int *new_qtl_genotype, double *modified_parms, 
	    double *new_effect_parms, double *prob,
	    double *work_qtl, double *test_o, double *temp_qtl_geno,
	    double *work_trait, double *tau, double *test_tau,
	    double *work, double *test_effect, double *up_tri){

  int    j, temp_qtl, new_interval;
  double log_accept_birth_prob, new_position;
  double uni_ran, temp_mu;
  /*
  int    *new_qtl_genotype = (int *) malloc( MAXTRAIT * sizeof(int) );
  double *modified_parms = (double *) malloc(MAXQTL*sizeof(double));
  double *new_effect_parms = (double *) malloc(2*sizeof(double));
  */

  log_accept_birth_prob = 0.0;
  temp_qtl = *nqtl;
  temp_mu = *mu;

/* GET NEW BIRTH INTERVAL */

  new_interval = get_new_interval( nmark, temp_qtl, &lambda[0] );

/* GET POSITION FOR NEW QTL WITHIN NEW INTERVAL */

  new_position = get_new_locus(new_interval);

/* GET GENOTYPES AT NEW POSITION */

  new_qtl_genotype = get_new_qtl_genotype(ntrait, &marker_genotype[0][0],
			  &dist[0], &recombination[0], new_position,
			  &new_qtl_genotype[0], &prob[0]);

/* GET NEW EFFECT */

  modified_parms[0] = temp_mu;
  for(j=0;j<temp_qtl;j++)
    modified_parms[j+1] = effect[j];

  get_birth_effect((long int) ntrait, (long int) temp_qtl, 
		  &qtl_genotype[0][0], 
		  &new_qtl_genotype[0],
		  &y[0], &modified_parms[0], 
		  &new_effect_parms[0], sigmasq,
		  &work_qtl[0], &test_o[0], &temp_qtl_geno[0],
		  &work_trait[0], &tau[0], &test_tau[0], &work[0],
		  &test_effect[0], &up_tri[0]);

/* DECIDE WHETHER OR NOT TO ACCEPT NEW QTL */

/************************************************
  If jacobian = 0, then acceptance probability 
  is 0.
************************************************/

  if( new_effect_parms[1] > 0.0000001 ){

    log_accept_birth_prob = accept_birth(ntrait, nmark, temp_qtl, &y[0],
		        temp_mu, &effect[0], sigmasq, &modified_parms[0],
		        &qtl_genotype[0][0], &new_qtl_genotype[0],
		        &marker_genotype[0][0], &dist[0], &cumdist[0],
		        &recombination[0], &lambda[0], new_position, 
		        mu_prior_mn, mu_prior_sd, 
		        effect_prior_mn, effect_prior_sd, 
		        sig_prior_a1, sig_prior_a2, qtl_prior_mn, 
		        new_effect_parms[0], new_effect_parms[1],
			&prob[0]);

    log_accept_birth_prob = log_accept_birth_prob +
			    log(qtl_prior_mn) - log(temp_qtl+1);
    uni_ran = genunf(0.0,1.0);
    uni_ran = log(uni_ran);
/*
printf("birth log uni ran is %f\n", uni_ran);
printf("log birth accept prob is %f\n", log_accept_birth_prob);
*/
    if(uni_ran < log_accept_birth_prob){
      nmove[1] = nmove[1] + 1; 
/*
printf("accepted birth\n");
printf("\n");
*/
      temp_mu = modified_parms[0];
      for(j=0;j<temp_qtl;j++)
        effect[j] = modified_parms[j+1];
      *mu = temp_mu;

      if(*nqtl == 0){
        lambda[0] = new_position;
        effect[0] = modified_parms[1];
        for(j=0; j<ntrait; j++)
	  qtl_genotype[j][0] = new_qtl_genotype[j];
        *mu = modified_parms[0];
        *nqtl = *nqtl+1;
      }
      else{
      setvals(ntrait, nqtl, mu, &effect[0], &lambda[0], &modified_parms[0],
	      new_position, &qtl_genotype[0][0], &new_qtl_genotype[0]);
      }
    }
  }
  /* else printf("rejected birth\n"); */
/*
  free (modified_parms);
  free (new_qtl_genotype);
  free (new_effect_parms);
*/
}
