/****************************************************
  File:       death.c
  Written by: Jaya Satagopan (satago@biost.mskcc.org)
  Date:       March 9, 1998

  Purpose:

    Function DEATH performs the "death step" for
    removing an existing QTL.
****************************************************/

#include "revjump.h"

extern float  genunf(float, float);
extern float  genunf(float, float);
extern double dist_rec(double, int, double *);
extern void   f1_prob(int, int, double, double, double *);
extern int    get_death_interval(int, int );
extern int    *get_new_qtl_genotype(int, int *, double *, double *, double);
extern double get_new_locus(int);
extern void   get_death_effect(long int, long int, long int, 
                             int *, double *, double *, double *, double,
			     double *, double *, double *, double *,
			     double *, double *, double *);
extern double accept_birth(int, int, int, double *, double, double *, double,
	double *, int *, int *, int *, double *, double *, double *,
	double *, double, double, double, double, double, double, double, 
	double, double, double, double *);
extern void setvals(int, int *, double *, double *, double *, double *,
	double, int *, int *);

void death( int ntrait, int nmark, int *nqtl, double *y,
	    double *mu, double *effect, double sigmasq,
	    int qtl_genotype[][MAXQTL], int marker_genotype[][MAXMARKER],
	    double *dist, double *cumdist, double *recombination,
	    double *lambda, double mu_prior_mn, double mu_prior_sd,
	    double effect_prior_mn, double effect_prior_sd,
	    double sig_prior_a1, double sig_prior_a2, double qtl_prior_mn,
	    int *nmove,
	    int *new_qtl_genotype, double *modified_parms,
	    double *new_death_values,
	    double *work_qtl, double *temp_qtl_geno, double *work_trait,
	    double *tau, double *work, double *up_tri, 
	    double *test_effect, double *prob,
	    double *birth_parms, double *death_parms, int *drop_qtl_geno,
	    int death_qtl_geno[][MAXQTL], double *death_lambda){

  int    i, j, temp_qtl, death_interval;
  double log_accept_death_prob, new_position;
  double uni_ran, temp_mu;
  /*
  int    *new_qtl_genotype = (int *) malloc( MAXTRAIT * sizeof(int) );
  double *modified_parms = (double *) malloc(MAXQTL*sizeof(double));
  double *new_death_values = (double *) malloc(2*sizeof(double));
  double *birth_parms = (double *) malloc(MAXQTL * sizeof(double));
  double *death_parms = (double *) malloc(MAXQTL * sizeof(double));
  int *drop_qtl_geno  = (int *) malloc(MAXTRAIT * sizeof(int));
  int death_qtl_geno[MAXTRAIT][MAXQTL];
  double *death_lambda = (double *) malloc(MAXQTL * sizeof(double));
  */
  double drop_lambda;

  temp_mu = *mu;
  log_accept_death_prob = 0.0;
  temp_qtl = *nqtl;

/* GET DEATH INTERVAL */

  death_interval = get_death_interval( nmark, temp_qtl );

/* GET DEATH EFFECTS */

/*
for(j=0; j<temp_qtl; j++)
  printf("effect %d is %f\n", j, effect[j]);
*/

  modified_parms[0] = temp_mu;
  for(j=0;j<temp_qtl;j++){
    modified_parms[j+1] = effect[j];
  }

  get_death_effect( (long int) ntrait, (long int) temp_qtl, 
                    (long int) death_interval,
		    &qtl_genotype[0][0], &y[0], &modified_parms[0],
		    &new_death_values[0], sigmasq,
		    &work_qtl[0], &temp_qtl_geno[0], &work_trait[0],
		    &tau[0], &work[0], &up_tri[0], &test_effect[0]);

  drop_lambda = lambda[death_interval];
  for(i=0; i<ntrait; i++)
    drop_qtl_geno[i] = qtl_genotype[i][death_interval];

  if(death_interval == 0){
    for(j=1; j<temp_qtl; j++){
      death_lambda[j-1] = lambda[j];
      for(i=0; i<ntrait;i++)
	death_qtl_geno[i][j-1] = qtl_genotype[i][j];
    }
  }
  else{
    if(death_interval == temp_qtl){
      for(j=0; j<(temp_qtl-1); j++){
	death_lambda[j] = lambda[j];
        for(i=0; i<ntrait; i++)
	  death_qtl_geno[i][j] = qtl_genotype[i][j];
      }
    }
    else{
      for(j=0; j<death_interval; j++){
	death_lambda[j] = lambda[j];
        for(i=0; i<ntrait; i++)
	  death_qtl_geno[i][j] = qtl_genotype[i][j];
      }
      for(j=(death_interval+1); j<temp_qtl; j++){
	death_lambda[j-1] = lambda[j];
        for(i=0; i<ntrait; i++)
	  death_qtl_geno[i][(j-1)] = qtl_genotype[i][j];
      }
    }
  }

  birth_parms[0] = temp_mu;
  for(i=1; i<=temp_qtl; i++)
    birth_parms[i] = effect[i-1];
  for(i=0; i<(temp_qtl-1); i++)
    death_parms[i] = modified_parms[i+1];

/* DECIDE WHETHER OR NOT TO ACCEPT NEW QTL */

/**************************************************
proceed with accept/reject death only if jacobian
is not 0. if jacobian is 0, reject death.
**************************************************/

  if( new_death_values[1] > 0.0000001 ){
    log_accept_death_prob = accept_birth(ntrait, nmark, (temp_qtl-1), &y[0],
			    modified_parms[0], &death_parms[0], sigmasq,
			    &birth_parms[0], &death_qtl_geno[0][0],
			    &drop_qtl_geno[0], &marker_genotype[0][0],
		            &dist[0], &cumdist[0],
		            &recombination[0], &death_lambda[0],
			    drop_lambda,
		            mu_prior_mn, mu_prior_sd, 
		            effect_prior_mn, effect_prior_sd, 
		            sig_prior_a1, sig_prior_a2, qtl_prior_mn, 
		            new_death_values[0], new_death_values[1],
			    &prob[0]);

    log_accept_death_prob = log_accept_death_prob + 
			    log(qtl_prior_mn) - log(temp_qtl);
    log_accept_death_prob = -log_accept_death_prob;

/*printf("death acceptance probability is %f\n", log_accept_death_prob);  */

    uni_ran = genunf(0.0,1.0);
    uni_ran = log(uni_ran);
    if(uni_ran < log_accept_death_prob){ 
      nmove[3] = nmove[3] + 1; 
      *mu = modified_parms[0];
      for(i=0; i<(temp_qtl-1); i++){
        effect[i] = modified_parms[i+1];
        lambda[i] = death_lambda[i];
        for(j=0; j<ntrait; j++)
          qtl_genotype[j][i] = death_qtl_geno[j][i];
        }
        *nqtl = temp_qtl - 1;
    }
  }
  /* else printf("rejected death\n"); */
}
