/**************************************************************
  File:         geno_prob.c
  Written by:   Jaya Satagopan (satago@biost.mskcc.org)
  Date:         August 29, 1997

  Purpose:
  -------

    This function gets the probability of genotype at a 
    (marker or qtl) locus given its location and flanking 
    marker genotypes for a F1 derived double haploid population.

**************************************************************/

#include "revjump.h"

void f1_prob(int flank_mark1, int flank_mark2, 
		double r1, double R, double *prob ){

  double r2;
  if(r1 < 0.5) r2 = (R-r1)/(1-2*r1);
  else r2 = 0.5;
  prob[0] = 0.0;
  switch(flank_mark1){
    case 1:{
      switch(flank_mark2){
	case 1:{
	  prob[0] = 1-r1*r2/(1-R);
	  break;
	}
	default:{
	  prob[0] = r2*(1-r1)/R;
	  break;
	}
      }
      break;
    }
    default :{
      switch(flank_mark2){
	case 1:{
	  prob[0] = r1*(1-r2)/R;
	  break;
	}
	default:{
	  prob[0] = r1*r2/(1-R);
	  break;
	}
      }
      break;
    }
  }
  prob[1] = 1.0 - prob[0];
}

/***********************************************
              FUNCTION HALREC
***********************************************/

double halrec(double r1, double r2){
  double R;
  R = r1+r2-2*r1*r2;
  return R;
}
