/****************************************************
  File:       get_death_parms.c
  Written by: Jaya Satagopan (satago@biost.mskcc.org)
  Date:       March 9, 1998

  Purpose:

****************************************************/

#include "revjump.h"

extern float genunf(float, float);
extern int get_death_interval(int, int );

/* LAPACK ROUTINES */
extern int dgeqrf_(long int *, long int *, double *, long int *,
		   double *, double *, long int *, long int *);
extern int dtrtrs_(char *, char *, char *, long int *, long int *,
                   double *, long int *, double *, long int *,  long int *);
extern int dormqr_(char *, char *, long int *, long int *, long int *,
	           double *, long int *, double *, double *,
		   long int *, double *, long int *, long int *);

/***************************************
      Function GET_DEATH_INTERVAL
    randomly draws a new QTL interval 
***************************************/

int get_death_interval( int nmark, int nqtl ){
  int death_interval;
  double uni_ran, work;

  work = nqtl;
  uni_ran = genunf(0.0, 1.0);
  work = work * uni_ran;
  death_interval = work;

  return death_interval;
}

/***************************************
      Function GET_DEATH_EFFECT
        get modified effects
***************************************/
void get_death_effect(long int ntrait, long int nqtl, long int drop_int,
    int qtl_genotype[][MAXQTL], double *y, double *modified_parms,
    double *new_death_values, double sigmasq,
    double *work_qtl, double *temp_qtl_geno, double *work_trait, 
    double *tau, double *work, double *up_tri, double *test_effect){

  char side, trans, uplo, diag;
  long int i, j, info, ncol_q;
  long int lwork = nqtl-1;
  long int nrhs;
  double scalar_v, opq_sq, opq_prime_opy, qpy, qpq;
  double mu, drop_effect;

/* double *work_qtl = (double * malloc( (MAXTRAIT * MAXQTL) * sizeof(double) ); 
  double *temp_qtl_geno = (double *) malloc( MAXTRAIT*sizeof(double) );
  double *work_trait = (double *) malloc( MAXTRAIT*sizeof(double) );
  double *tau = (double *) malloc( MAXQTL * sizeof(double) );
  double *work = (double *) malloc( MAXQTL*sizeof(double) );
  double *up_tri = (double *) malloc( (MAXQTL*MAXQTL) *sizeof(double) );
  double *test_effect = (double *) malloc( MAXQTL * sizeof(double) );
*/


  mu = modified_parms[0];

  if(drop_int == 0){
    for(j=2;j<=nqtl;j++)
      test_effect[(j-2)] = modified_parms[j];
  }
  else{
    if(drop_int == (nqtl-1)){
      for(j=1; j<=(nqtl-1); j++)
	test_effect[(j-1)] = modified_parms[j];
    }
    else{
      for(j=0; j<(drop_int-1); j++)
	test_effect[j] = modified_parms[j+1];
      for(j=(drop_int+1); j<=nqtl; j++)
	test_effect[(j-2)] = modified_parms[j];
    }
  }
  drop_effect = modified_parms[drop_int+1];

  for(i=0;i<ntrait;i++){
    work_qtl[i] = 1.0;
    temp_qtl_geno[i] = qtl_genotype[i][drop_int];
    work_trait[i] = y[i];
  }
  for(j=0; j<drop_int; j++){
    for(i=0; i<ntrait; i++)
      work_qtl[ntrait+i+j*ntrait] = qtl_genotype[i][j];
  }
  for(j=(drop_int+1); j<nqtl; j++){
    for(i=0; i<ntrait; i++)
      work_qtl[ntrait+i+(j-1)*ntrait] = qtl_genotype[i][j];
  }

/* Get OU decomposition of work_qtl, where the columns of O
   are orthogonal and U is upper-triangular */

  info = dgeqrf_(&ntrait, &nqtl, &work_qtl[0], &ntrait, 
		 &tau[0], &work[0], &nqtl, &info);

/* Get O'Q, where Q = temp_qtl_geno are the QTL genotypes
   of the candidate QTL to be dropped */

  side = 'L';
  trans = 'T';    /* side and trans say that we need O'Q */
  ncol_q = 1;     /* says that Q is a column vector */

  info = dormqr_(&side, &trans, &ntrait, &ncol_q, &nqtl,
		 &work_qtl[0], &ntrait, &tau[0], &temp_qtl_geno[0],
		 &ntrait, &work[0], &nqtl, &info);

  qpq = 0.0;
  opq_sq = 0.0;
 for(j=0; j<nqtl; j++) 
/*  for(j=0; j<lwork; j++)  */
/*  for(j=0; j<ntrait; j++) */
    opq_sq = opq_sq + temp_qtl_geno[j] * temp_qtl_geno[j];
  for(j=0; j<ntrait; j++)
    qpq = qpq + qtl_genotype[j][drop_int] * qtl_genotype[j][drop_int];

/* CREATE THE UPPER TRIANGULAR MATRIX UP_TRI */	

  for(i=0; i<nqtl; i++)
    for(j=0; j<=i; j++)
      up_tri[j+nqtl*i] = work_qtl[j+ntrait*i];

/* NEED TO COMPUTE (X'X)^{-1} X'Q TYPE OPERATION.
   CAN USE DTRTRS FUNCTION FROM LAPACK. NEED TO DEFINE
   UPLO, TRANS AND DIAG TO TELL DTRTRS WHAT IT MUST DO.
   THESE ARE DEFINED FIRST AND DTRTRS FUNCTION IS THEN
   USED BELOW. */

  uplo = 'U';
  trans = 'N';
  diag = 'N';
  nrhs = 1;

  info = dtrtrs_(&uplo, &trans, &diag, &nqtl, &nrhs, &up_tri[0],
		 &nqtl, &temp_qtl_geno[0], &nqtl, &info);

  for(i=0; i<nqtl; i++)
    if(temp_qtl_geno[i] < -999) temp_qtl_geno[i] = 0.0;

  mu = mu + temp_qtl_geno[0] * drop_effect;

if(nqtl>1){
  for(j=0; j<(nqtl-1); j++){
    test_effect[j] = test_effect[j] + temp_qtl_geno[j+1] * drop_effect;
  }
}
else
  test_effect[j] = 0.0;

  modified_parms[0] = mu;
  if(nqtl>1){
    for(j=0; j<(nqtl-1); j++)
      modified_parms[j+1] = test_effect[j];
  }

  modified_parms[nqtl] = 0.0;

new_death_values[0] = drop_effect;
new_death_values[1] = qpq - opq_sq;

/*
  free (work_qtl);
  free (temp_qtl_geno);
  free (work_trait);
  free (tau);
  free (work);
  free (up_tri);
  free (test_effect);
*/

}
