/***********************************************
  File:         get_new_lambda.c
  Date:         September 8, 1997
  Written by:   Jaya Satagopan (satago@biost.mskcc.org)

  Purpose:

    The function get_new_lambda simulates
    new lambda values from a uniform distribution.

***********************************************/

#include "revjump.h"

extern float genunf(float, float);
extern void sort_val( double *, int, int );
extern void swap( double *, int, int );
extern int numcmp(const void *, const void *);

double *get_new_lambda(int nqtl, int nmark, double *lambda,
                       double *new_lambda){
  double low, high;
  int iqtl;
  double temp;

  for(iqtl=0; iqtl<nqtl; iqtl++){
    low = lambda[iqtl] - 0.5;
    high = lambda[iqtl] + 0.5;
    if( low < 0.0 ) low = 0.0;
    if( high > (nmark-1) ) high = nmark-1;
    temp = genunf(0.0, 1.0);
    temp = temp * (high - low) + low;
    new_lambda[iqtl] = temp; 
    if( new_lambda[iqtl] < 0.01 )
      new_lambda[iqtl] = 0.01;
  }

  if( nqtl > 1 )
     qsort(&new_lambda[0], nqtl, sizeof(double), &numcmp);

  return new_lambda;
}

/****************************************
      Function SORT_VAL

      Date: September 8, 1997
****************************************/

void sort_val( double *new_lambda, int left, int right){
  int i, last;
  if( left >= right ) return;
  swap( &new_lambda[0], left, (left+right)/2 );
  last = left;
  for(i=left+1; i<=right; i++)
    if( new_lambda[i] < new_lambda[left] )
      swap( &new_lambda[0], ++last, i );
  swap( &new_lambda[0], left, last );
  sort_val( &new_lambda[0], left, last-1 );
  sort_val( &new_lambda[0], last+1, right );
}

/****************************************
      Function SWAP

      Date: September 8, 1997
****************************************/

void swap( double *new_lambda, int i, int j ){
  double temp[1];
  temp[0] = new_lambda[i];
  new_lambda[i] = new_lambda[j];
  new_lambda[j] = temp[0];
}
