/*****************************************************
  File:        lam_accept.c
  Date:        September 8, 1997
  Written by:  Jaya Satagopan (satago@biost.mskcc.org)

  Purpose:
    Calculate the metropolis-hastings acceptance
    probability for lambda versus new_lambda.
    Return whichever value of lambda is accepted.
*****************************************************/

#include "revjump.h"

extern double dist_rec(double, int, double *);
extern float genunf(float, float);
extern void set_miss_marker(int, int, int, double *, int *, int *);
extern void get_accept_prob( int, int, int, int, int *, int *,
                               int, int, int, int, double *,
                               double, double, double, double,
                               double *, double *, double *);
extern void which_marker(int, int, int, int *, int *, int, double *,
			 double *, int, int, int *); 
extern void update_qtl_geno(int, int, int, int, int, int, 
			    int *, double, double, int, 
			    double *, double *, double *, double,
			    double *, double, int *, double *,
			    int *, double *, double *);

void get_lambda_qtl(int ntrait, int nmark, int nqtl, double *lambda,
                      double *new_lambda, double *dist,
                      double *recombination,
                      int miss_mark[][MAXMARKER],
                      int marker_genotype[][MAXMARKER],
                      int qtl_genotype[][MAXQTL],
		      double *y, double *ymiss, double mu,
		      double *effect, double sigmasq,
		      int movetype_lambda, int *nmove, int *nlambda,
		      double *sum_val, int *markers, double *qival,
		      double *qprob_val){
  int iqtl, dist_old, dist_new, flank_right, flank_left;
  double r1_old, R_old, r1_new, R_new, work, uni_ran;
  double accept_prob[1];
  int i,j;

  if( nqtl > 1 ) flank_right = lambda[1];
  else flank_right = 1;

  for(iqtl=0; iqtl<nqtl; iqtl++){
    dist_old = lambda[iqtl];
    work = lambda[iqtl];
    r1_old = dist_rec(work, dist_old, &dist[0]);
    R_old = recombination[dist_old];

    dist_new = new_lambda[iqtl];
    flank_left = dist_new;
    work = new_lambda[iqtl];
    r1_new = dist_rec(work, dist_new, &dist[0]);
    R_new = recombination[dist_new];

    get_accept_prob(ntrait, nmark, nqtl, iqtl,
                  &marker_genotype[0][0], &qtl_genotype[0][0],
                  dist_old, dist_new, flank_left, flank_right,
                  &dist[0], r1_old, R_old, r1_new, R_new,
                  &lambda[0], &new_lambda[0], &accept_prob[0]);

    uni_ran = genunf(0.0, 1.0);
    if( uni_ran < accept_prob[0] ){
      lambda[iqtl] = new_lambda[iqtl];
      dist_old = dist_new;
      r1_old = r1_new;
      R_old = R_new;
      if(movetype_lambda == 3)
        nmove[5] = nmove[5]+1;
      *nlambda = *nlambda + 1;
    }
    flank_left = lambda[iqtl];

    /* UPDATE MISSING MARKER GENOTYPES */
    if( miss_mark[0][flank_left] > 0 ){
      set_miss_marker(ntrait, nmark, flank_left, &recombination[0],
                      &miss_mark[0][0], &marker_genotype[0][0]);
    }
    if( miss_mark[0][flank_right] > 0 ){
      set_miss_marker(ntrait, nmark, flank_right, &recombination[0],
                      &miss_mark[0][0], &marker_genotype[0][0]);
    }

    update_qtl_geno(ntrait, nqtl, nmark, iqtl, flank_left, flank_right,
                    &marker_genotype[0][0], r1_old, R_old,
                    dist_old, &lambda[0], &new_lambda[0], &y[0],
                    mu, &effect[0], sigmasq, &qtl_genotype[0][0],
		    &sum_val[0], &markers[0], &qival[0], &qprob_val[0]);
    flank_right = lambda[iqtl];

  }
}
