/****************************************************
  File:       likeli_ratio.c
  Written by: Jaya Satagopan (satago@biost.mskcc.org)
  Date:       October 14, 1997

  Purpose:

    Function ACCEPT_BIRTH computes the acceptance
    probability for the birth of a new QTL.
****************************************************/

#include "revjump.h"

extern double dist_rec(double, int, double *);

/***************************************
    Function GET_LOG_MODEL_RATIO

  computes the ratio of the larger 
  model over the smaller model
***************************************/

double get_log_model_ratio(int ntrait, int nqtl, double *y, 
		      double *modified_parms, double mu,
		      double *effect, double sigmasq,
		      int qtl_genotype[][MAXQTL], int *new_qtl_genotype){
  double small_sum1, small_sum2, large_sum1, large_sum2;
  double mod_mu, log_model_ratio;
  int i,j;

  mod_mu = modified_parms[0];
  /* for(i=0;i<nqtl;i++){
    mod_effect[i] = modified_parms[i+1];
  } */

  large_sum1 = 0.0;
  large_sum2 = 0.0;

  for(i=0;i<ntrait;i++){
    small_sum1 = 0.0;
    small_sum2 = 0.0;

    for(j=0;j<nqtl;j++){
      small_sum1 = small_sum1 + modified_parms[(j+1)] * qtl_genotype[i][j];
      small_sum2 = small_sum2 + effect[j] * qtl_genotype[i][j];
    }
    small_sum1 = small_sum1 + modified_parms[nqtl+1] * new_qtl_genotype[i];
    small_sum1 = y[i] - mod_mu - small_sum1;
    small_sum1 = small_sum1 * small_sum1;

    small_sum2 = y[i] - mu - small_sum2;
    small_sum2 = small_sum2 * small_sum2;

    large_sum1 = large_sum1 + small_sum1;
    large_sum2 = large_sum2 + small_sum2;
  }

  log_model_ratio = (large_sum2 - large_sum1) / (2.0 * sigmasq);

 /* printf("log model ratio is %f\n", log_model_ratio);  */

  return log_model_ratio;
}

/*********************************************
*********************************************/

double get_qtl_ratio(int ntrait, int nmark, int nqtl,
    int qtl_genotype[][MAXQTL], int *new_qtl_genotype,
    int marker_genotype[][MAXMARKER], double *dist,
    double *recombination, double *lambda, double new_position,
    double *pival){

  int j, markold;
  double dval, r1old, Rold, log_qtl_ratio;
  /* double *pival = (double *) malloc(2*sizeof(double)); */

  log_qtl_ratio = 0.0;
  for(j=0;j<ntrait;j++){
    markold = new_position;
    r1old = dist_rec(new_position, markold, &dist[0]);
    Rold = recombination[markold];

    f1_prob( marker_genotype[j][markold], marker_genotype[j][markold+1],
	     r1old, Rold, &pival[0] );
    if( new_qtl_genotype[j] == 1 ){ 
      if( pival[0] != 0 )
        log_qtl_ratio = log_qtl_ratio + log(pival[0]);
    }
    else{ 
      if( pival[1] != 0 )
        log_qtl_ratio = log_qtl_ratio + log(pival[1]);
    }
  }

/* printf("log qtl ratio is %f\n", log_qtl_ratio); */

  return log_qtl_ratio;
}

/*****************************************************
*****************************************************/


double get_position_ratio(int nqtl, int nmark, double *dist){
  
  double log_position_ratio;

/* printf("distance is %f\n", dist[nmark-1]); */

  log_position_ratio = log(1+nqtl) - log(dist[nmark-1]);  

/* printf("log position ratio is %f\n", log_position_ratio); */

  return log_position_ratio;
}

/*****************************************************
*****************************************************/

double get_par_ratio(int nqtl, double mu, double *effect,
	double *modified_parms, double mu_prior_mn, double mu_prior_var,
	double effect_prior_mn, double effect_prior_var){

  int i;
  double temp1, temp2, temp3, temp4, log_parms_ratio;
  double mod_mu, new_effect;
  double twopi = 6.283;

  temp1 = 0.0;
  temp2 = 0.0;
  temp3 = 0.0;
  temp4 = 0.0;
  mod_mu = modified_parms[0];

  /* for(i=0;i<nqtl;i++)
    mod_effect[i] = modified_parms[i+1]; */

  new_effect = modified_parms[nqtl+1];

  for(i=0;i<nqtl;i++){
    temp3 = modified_parms[(i+1)] - effect_prior_mn;
    temp3 = temp3 * temp3 / (2 * effect_prior_var);
    temp2 = temp2 + temp3;
  }
  temp3 = mod_mu - mu_prior_mn;
  temp2 = temp2 + temp3 * temp3 / (2 * mu_prior_var);
  temp3 = new_effect - effect_prior_mn;
  temp2 = temp2 + temp3 * temp3 / (2 * effect_prior_var);

  for(i=0;i<nqtl;i++){
    temp3 = effect[i] - effect_prior_mn;
    temp3 = temp3 * temp3 / (2 * effect_prior_var);
    temp4 = temp4 + temp3;
  }
  temp3 = mu - mu_prior_mn;
  temp4 = temp4 + temp3 * temp3 / (2 * mu_prior_var);

  temp1 = temp2 - temp4;
  temp1 = -temp1;

  log_parms_ratio = -1/2 * log(twopi * effect_prior_var) + temp1;

  return log_parms_ratio;
}

/*****************************************************
*****************************************************/

double get_qtl_number_ratio( double qtl_prior_mn, int nqtl){
  double log_qtl_number_ratio;

  log_qtl_number_ratio = log(qtl_prior_mn) - log(nqtl+1);

  return log_qtl_number_ratio;
}

/*****************************************************
*****************************************************/

double get_q_birth(int ntrait, int nmark, int nqtl, int *new_qtl_genotype,
    int marker_genotype[][MAXMARKER], double new_position,
    double *dist, double *cumulative_dist, double *recombination,
    double effect_uniform_number, double *pi_val){

  double twopi = 6.243;
  int j, markold;
  double dnmark, dnqtl, dtemp, temp1, temp2, temp3, r1, r2, R, log_q_birth;
  /* double *pi_val = (double *) malloc(2*sizeof(double)); */

  markold = new_position;
  temp1 = 0.0;
  temp2 = 0.0;
  temp3 = 0.0;
  dnmark = nmark;
  dnqtl = nqtl;

  dtemp = dnmark - dnqtl - 1;
  if(dtemp == 0) dtemp = 1;

/*
printf("new position is %f\n", new_position);
printf("cum distances are %f and %f\n", cumulative_dist[markold],
  cumulative_dist[markold+1]);
*/

  temp1 = -log(dtemp) - 
	   log( cumulative_dist[markold+1]/100 
		- cumulative_dist[markold]/100 );
  temp3 = effect_uniform_number * effect_uniform_number;
  temp3 = -0.5 * log(twopi) - temp3 / 2.0;

  r1 = dist_rec(new_position, markold, &dist[0]);
  R  = recombination[markold];


  if(r1 < 0.5) r2 = (R - r1) / (1 - 2*r1);
  else r2 = 0.5;

  for(j=0;j<ntrait;j++){
    pi_val[0] = 0.0;
    pi_val[1] = 0.0;

    if( marker_genotype[j][markold] == 1 ){
      if( marker_genotype[j][1+markold] == 1 )
	pi_val[0] = (1-r1) * (1-r2) / (1-R);
	/* 1 - r1*r2 / (1-R); */
      else 
	pi_val[0] = r2*(1-r1)/ R;
    }
    else{
      if( marker_genotype[j][1+markold] == 1 )
	pi_val[0] = r1 * (1-r2) / R;
      else
	pi_val[0] = r1*r2/(1-R);
    }
    pi_val[1] = 1.0 - pi_val[0];

    if( new_qtl_genotype[j] == 1) 
      if( pi_val[0] != 0 )
        temp2 = temp2 + log(pi_val[0]);
    else
      if( pi_val[1] != 0 )
        temp2 = temp2 + log(pi_val[1]);
  }
  log_q_birth = temp1 + temp3 + temp2;

/*printf("temp 1 is %f\n", temp1);
printf("temp 2 is %f\n", temp2);
printf("temp 3 is %f\n", temp3);*/
  return log_q_birth;
}

/****************************************
****************************************/

double get_q_death(int nqtl){
  double log_q_death;

  log_q_death = - log(nqtl+1);
  return log_q_death;
}

/********************************************
********************************************/

double get_jacobian(double sigmasq, double V_val){
  double log_jacobian;

 /*printf("v val is %f\n", V_val); */

  if( V_val <= 0.00001)
  {
    log_jacobian = 0.0;
  }
  if(V_val > 0.00001)
  {
    log_jacobian = 0.5 * log(V_val) - 0.5 * log(sigmasq);
  }

/* printf("log jacobian is %f\n", log_jacobian); */

  return log_jacobian;
}
