/**************************************************************
  File:         main.c
  Written by:   Jaya Satagopan (satago@biost.mskcc.org)
  Date:         August 26, 1997

  Purpose:
  -------

    This is the MAIN program for reversible jump mcmc
    computations.

**************************************************************/

#include "revjump.h"

extern void getdata( int *, double *, int *, double *,
		     double *, double *, double *);
extern void getcumdist_recom( int, double *, double *, double * );
extern void findmiss_marker( int, int, int *, int * );
extern void findmiss_trait( int , double *, double * );
extern void initmiss_marker( int, int, double *, int *, int * );
extern void f1_prob(int, int, double, double, double *);
extern void initqtl_geno(int, int, int, double *, double *, double *,
			 int *, int * );
extern double dist_rec( double, int, double * );
extern double haldane( double );
extern float genunf(float, float);
extern double halrec(double, double);
extern void mcmc(int, int , int , int , double *, double *,
		 int *, int *, int *,
		 double *, double *, double *, double *,
		 double , double *, double , 
		 double *, int , int );

main()
{
  int nvals[6];
  int revjump, nmark, ntrait, niter, nby, nqtl; 
  double cumdist[MAXMARKER], dist[MAXMARKER], 
	 recombination[MAXMARKER], y[MAXTRAIT], ymiss[MAXTRAIT];
  int marker_genotype[MAXTRAIT][MAXMARKER], miss_mark[MAXTRAIT][MAXMARKER+1],
      qtl_genotype[MAXTRAIT][MAXQTL];
  double lambda[MAXQTL], model_parms[MAXQTL+2];
  int i,j;
  double mu, sigmasq, effect[MAXQTL];
  double uni_ran;
  double hyper_parms[7];

  setall(time(NULL), time(NULL)+1);

  getdata( &nvals[0], &y[0], &marker_genotype[0][0], 
	   &dist[0], &model_parms[0], &lambda[0], &hyper_parms[0] );

  revjump = nvals[0];
  ntrait = nvals[1];
  nmark  = nvals[2];
  niter  = nvals[3];
  nby    = nvals[4];
  nqtl   = nvals[5];
  mu = model_parms[0];

  sigmasq = model_parms[1];
  if( nqtl > 0 ){
    for(i = 0; i<nqtl; i++ )
      effect[i] = model_parms[(i+2)];
  }

  getcumdist_recom( nmark, &dist[0], &cumdist[0], &recombination[0] );
  findmiss_marker( ntrait, nmark, &marker_genotype[0][0], &miss_mark[0][0] );
  findmiss_trait( ntrait, &y[0], &ymiss[0] );
  initmiss_marker( ntrait, nmark, &recombination[0], &miss_mark[0][0],
		   &marker_genotype[0][0]);
  initqtl_geno(ntrait, nmark, nqtl, &lambda[0], &dist[0], 
	       &recombination[0], &marker_genotype[0][0], 
	       &qtl_genotype[0][0]);

  mcmc( revjump, ntrait, nmark, nqtl, &y[0], &ymiss[0],
	&marker_genotype[0][0], &miss_mark[0][0], 
	&qtl_genotype[0][0],
	&dist[0], &recombination[0],&cumdist[0], 
	&lambda[0], mu, &effect[0], sigmasq, 
	&hyper_parms[0], niter, nby );
}
