/************************************************************
    File:        mcmc.c
    Date:        September 2, 1997
    Written by:  Jaya Satagopan (satago@biost.mskcc.org) 

    Purpose:

      Function MCMC gets the states of the required 
      Markov chain from its equilibrium distribution.
************************************************************/

#include "revjump.h"

extern void setmissy( int , int , double *, double , double *, double ,
                  int *, double *);
extern float genunf(float, float);
extern int select_move( int, int, double, int * );
extern double *get_new_lambda(int, int, double *, double *);
extern void get_lambda_qtl(int, int, int, double *, double *, double *, 
				     double *, int *, int *, int *,
				     double *, double *, double, 
				     double *, double, int, int *, int *,
				     double *, int *, double *, double *);
extern double get_new_mu(int, int, double *, int *, double *, 
			 double, double, double);
extern void *get_new_effect(int, int, double *, int *, double,
	  double *, double, double, double);
extern double get_new_sigmasq(int, int, double *, int *, double,
	  double *, double, double);

extern lam_to_dist(int, double *, double *, double *, double *);

extern void birth(int, int, int *, double *, double *, double *, double,
		  int *, int *, double *, double *, double *, double *,
		  double, double, double, double, double, double, 
		  double, int *, 
		  int *, double *, double *, double *,
		  double *, double *, double *, double *, double *,
		  double *, double *, double *, double *);

extern void get_accept_prob(int, int, int, int, int *, int *,
		  int, int, int, int, double *, double, double, 
		  double, double, double *, double *, double *);

extern void death(int, int, int *, double *, double *, double *, double,
		  int *, int *, double *, double *, double *, double *,
		  double, double, double, double, double, double, 
		  double, int *,
		  int *, double *, double *,
		  double *, double *, double *, double *,
		  double *, double *, double *, double *,
		  double *, double *, int *, int *, double *);


void mcmc( int revjump, int ntrait, int nmark, int nqtl, 
	   double *y, double *ymiss,
           int marker_genotype[][MAXMARKER], int miss_mark[][MAXMARKER],
           int qtl_genotype[][MAXQTL], 
           double *dist, double *recombination, double *cumdist,
           double *lambda, double mu, double *effect, double sigmasq,
           double *hyper_parms, int niter, int nby ){

  double mu_prior_mn, mu_prior_var, sigsq_prior_a1, sigsq_prior_a2,
  effect_prior_mn, effect_prior_var, qtl_prior_mn;
  int move_type, nlambda;
  int *nmove = (int *) malloc( 6 * sizeof(int) );

  int *new_qtl_genotype = (int *) malloc(MAXTRAIT * sizeof(int));
  int *markers = (int *) malloc( 2 * sizeof(int) );
  double *modified_parms = (double *) malloc(MAXQTL * sizeof(double));
  double *new_effect_parms = (double *) malloc(2 * sizeof(double));
  double *new_death_values = (double *) malloc(2 * sizeof(double));
  double *prob = (double *) malloc(2 * sizeof(double));
  double *work_qtl = (double *) malloc( (MAXTRAIT*MAXQTL)*sizeof(double ) );
  double *test_o = (double *) malloc( (MAXTRAIT*MAXQTL)*sizeof(double ) );
  double *temp_qtl_geno = (double *) malloc(MAXTRAIT*sizeof(double));
  double *work_trait = (double *) malloc(MAXTRAIT*sizeof(double));
  double *tau = (double *) malloc(MAXQTL*sizeof(double));
  double *test_tau = (double *) malloc(MAXQTL*sizeof(double));
  double *work = (double *) malloc(MAXQTL*sizeof(double));
  double *test_effect = (double *) malloc(MAXQTL*sizeof(double));
  double *up_tri = (double *) malloc( (MAXQTL*MAXQTL)*sizeof(double));
  double *sum_val = (double *) malloc( 2 * sizeof(double));
  double *qival = (double *) malloc(2 * sizeof(double));
  double *qprob_val = (double *) malloc(2 * sizeof(double));
  double *birth_parms = (double *) malloc(MAXQTL * sizeof(double));
  double *death_parms = (double *) malloc(MAXQTL * sizeof(double));
  double *death_lambda = (double *) malloc(MAXQTL * sizeof(double));
  int *drop_qtl_geno = (int *) malloc(MAXTRAIT * sizeof(int));
  int death_qtl_geno[MAXTRAIT][MAXQTL];

  FILE *write_res;
  FILE *write_nmove;
  FILE *write_lambda;

  int i, j, iqtl, iter, iobs, jmark, markold;
  double *new_lambda = (double *) malloc(MAXQTL * sizeof(double));
  double *lam_dist = (double *) malloc(MAXQTL * sizeof(double));
  int *num_qtl = (int *) malloc((MAXQTL+1) * sizeof(int));

  double diter, dnby, tempnby, dnbyval;
  int nbyval;

  write_res = fopen("result.write", "w");
  write_nmove = fopen("nmove.write", "w");
  write_lambda = fopen("nlambda.write", "w");

  mu_prior_mn = hyper_parms[0];
  mu_prior_var = hyper_parms[1];
  sigsq_prior_a1 = hyper_parms[2];
  sigsq_prior_a2 = hyper_parms[3];
  effect_prior_mn = hyper_parms[4];
  effect_prior_var = hyper_parms[5];
  qtl_prior_mn = hyper_parms[6];

  sigsq_prior_a1 = sigsq_prior_a1 + ntrait / 2.0;

  dnby = nby;

  nlambda = 0;
  /* for(i=0;i<3;i++)
    for(j=0;j<2;j++)
      nmove[i][j] = 0; */
  for(i=0; i<6; i++)
    nmove[i] = 0;

  for( iter=0; iter<niter; iter++){
    num_qtl[nqtl] = num_qtl[nqtl] + 1;
    if( ymiss[0] >0 )
      setmissy( ntrait, nqtl, &ymiss[0], mu, &effect[0], sigmasq,
           &qtl_genotype[0][0], &y[0] );

    /****************************************
      determine whether birth or death step
      or whether to update parameters only
    ****************************************/

/*********************************************************************
**********************************************************************
   BIRTH AND DEATH STEPS ARE REQUIRED ONLY WHEN DOING REVERSIBLE
   JUMP MCMC COMPUTATIONS, AS THE FOLLOWING IF STATEMENT INDICATES.

   FOR REVERSIBLE JUMP MCMC, CHOOSE MOVE TYPE (BIRTH, DEATH OR
   STAY) AND THEN PROCEED. 
**********************************************************************
*********************************************************************/

  if( revjump == 1 ){
    move_type = select_move( nmark, nqtl, qtl_prior_mn, &nmove[0] );

    if(move_type==1){ 

      birth(ntrait, nmark, &nqtl, &y[0], &mu, &effect[0], sigmasq,
	    &qtl_genotype[0][0], &marker_genotype[0][0],
	    &dist[0], &cumdist[0], &recombination[0], &lambda[0],
	    mu_prior_mn, mu_prior_var, effect_prior_mn, effect_prior_var,
	    sigsq_prior_a1, sigsq_prior_a2, qtl_prior_mn, &nmove[0],
	    &new_qtl_genotype[0], &modified_parms[0], 
	    &new_effect_parms[0], &prob[0],
	    &work_qtl[0], &test_o[0], &temp_qtl_geno[0],
	    &work_trait[0], &tau[0], &test_tau[0], &work[0],
	    &test_effect[0], &up_tri[0]);

     }

    if(move_type==2){

      death(ntrait, nmark, &nqtl, &y[0], &mu, &effect[0], sigmasq,
	    &qtl_genotype[0][0], &marker_genotype[0][0],
	    &dist[0], &cumdist[0], &recombination[0], &lambda[0],
	    mu_prior_mn, mu_prior_var, effect_prior_mn, effect_prior_var,
	    sigsq_prior_a1, sigsq_prior_a2, qtl_prior_mn, &nmove[0],
	    &new_qtl_genotype[0], &modified_parms[0], 
	    &new_death_values[0],
	    &work_qtl[0], &temp_qtl_geno[0], &work_trait[0], 
	    &tau[0], &work[0], &up_tri[0], &test_effect[0], &prob[0],
	    &birth_parms[0], &death_parms[0], &drop_qtl_geno[0],
	    &death_qtl_geno[0][0], &death_lambda[0]);
    }

    /*
    else{
      nmove[4] = nmove[4]+1;
    }
    */
  }

    /* REGULAR UPDATES BELOW */

    if(nqtl > 0)
    new_lambda = get_new_lambda(nqtl, nmark, &lambda[0], &new_lambda[0]);

/* METROPOLIS-HASTINGS STEP TO ACCEPT or REJECT LAMBDAs */

if(nqtl > 0)
    get_lambda_qtl(ntrait, nmark, nqtl, &lambda[0], &new_lambda[0],
		   &dist[0], &recombination[0], &miss_mark[0][0],
		   &marker_genotype[0][0], &qtl_genotype[0][0],
		   &y[0], &ymiss[0], mu, &effect[0], sigmasq,
		   move_type, &nmove[0], &nlambda, &sum_val[0],
		   &markers[0], &qival[0], &qprob_val[0]);

    mu = get_new_mu(ntrait, nqtl, &y[0], &qtl_genotype[0][0],
		    &effect[0], sigmasq, mu_prior_mn, mu_prior_var);

if(nqtl > 0)
    get_new_effect(ntrait, nqtl, &y[0], &qtl_genotype[0][0],
			    mu, &effect[0], sigmasq, 
			    effect_prior_mn, effect_prior_var);

    sigmasq = get_new_sigmasq(ntrait, nqtl, &y[0], &qtl_genotype[0][0],
			      mu, &effect[0], sigsq_prior_a1,
			      sigsq_prior_a2);

if(nqtl > 0)
    lam_to_dist(nqtl, &lambda[0], &cumdist[0], &dist[0], &lam_dist[0]);

for(iqtl=nqtl; iqtl<MAXQTL; iqtl++){
  effect[iqtl] = 0.0;
  lam_dist[iqtl] = 0.0;
}


/*
  printf("%d  %6.4f  ", nqtl, mu);
  for(iqtl=0; iqtl<nqtl; iqtl++)
    printf("%6.4f  ", effect[iqtl]);
  printf("%6.4f  ", sigmasq);
  for(iqtl=0; iqtl<nqtl; iqtl++)
    printf("%6.4f  ", lam_dist[iqtl]);
  printf("%d", move_type);
  printf("\n");
*/

/*   if( iter > 100000){ */
    diter = iter;
    tempnby = diter / dnby;
    nbyval = tempnby;
    dnbyval = nbyval;
    if( (tempnby - dnbyval) == 0 ){
      fprintf(write_res, "%d  %6.4f  ", nqtl, mu);
      for(iqtl=0; iqtl<nqtl; iqtl++)
        fprintf(write_res, "%6.4f  ", effect[iqtl]);
      fprintf(write_res, "%6.4f  ", sigmasq);
      for(iqtl=0; iqtl<nqtl; iqtl++)
        fprintf(write_res, "%6.4f  ", lam_dist[iqtl]);
      fprintf(write_res, "%d", move_type);
      fprintf(write_res,"\n");
      fflush(write_res);
    }
/*   } */
  }
  /*
  for(i=0; i<6; i++){
    printf("%d  ", nmove[i]);
  }
  */

  for(i=0; i<6; i++){
    fprintf(write_res, "%d  ", nmove[i]);
    fflush(write_res);
  }
  fprintf(write_res, "\n");
  for(i=0; i<=MAXQTL; i++){
    fprintf(write_res, "%d \n", num_qtl[i]);
    fflush(write_res);
  }
/*  printf("\n"); */
  fclose(write_res);

/*
  for(i=0; i<3; i++){
    fprintf(write_nmove, "%d    %d\n", nmove[i][0], nmove[i][1]);
    fflush(write_nmove);
  }
  for(i=0; i<MAXQTL; i++){
    fprintf(write_nmove, "%d \n", num_qtl[i]);
    fflush(write_nmove);
  }
  fclose(write_nmove);
  fprintf(write_lambda, "%d\n", nlambda);
  fclose(write_nmove);
  fclose(write_lambda);
  fclose(write_res);
*/
}
