/**************************************************
  File:       update_parms.c
  Date:       September 10, 1997
  Written by: Jaya M. Satagopan (satago@biost.mskcc.org)

  Purpose:

    Update the model parameters.
**************************************************/

#include "revjump.h"

extern float gennor(float, float);
extern float gengam(float, float);

/*****************************
    Function GET_NEW_MU
*****************************/

double get_new_mu(int ntrait, int nqtl, double *y,
                  int qtl_genotype[][MAXQTL], double *effect,
                  double sigmasq, double mu_prior_mn, double mu_prior_var){

  int iqtl, jobs;
  double work1, work2, mean0, var0, mu_val;

  var0 = (1.0/mu_prior_var) + (ntrait/sigmasq);
  work1 = 0.0;
  for(jobs=0;jobs<ntrait;jobs++){
    work2 = 0.0;
    for(iqtl=0;iqtl<nqtl;iqtl++)
      work2 = work2 + effect[iqtl] * qtl_genotype[jobs][iqtl];
    work1 = work1 + (y[jobs] - work2);
  }
  mean0 = (mu_prior_mn/mu_prior_var + work1/sigmasq) / var0;
  mu_val = gennor(0.0, 1.0);
  mu_val = mean0 + mu_val / sqrt(var0);
  return mu_val;
}

/*******************************
    Function GET_NEW_EFFECT
*******************************/

void *get_new_effect(int ntrait, int nqtl, double *y,
                       int qtl_genotype[][MAXQTL], double mu,
                       double *effect, double sigmasq,
                       double effect_prior_mn, double effect_prior_var){

  int iqtl, kqtl, jobs;
  double work, work1, work2, work3, var1, mean1;
  /* double *effect_val = (double *) malloc(nqtl * sizeof(double)); */

  for(iqtl=0;iqtl<nqtl;iqtl++){
    work3 = 0.0;

    for(jobs=0;jobs<ntrait;jobs++)
      work3 = work3 + qtl_genotype[jobs][iqtl] * qtl_genotype[jobs][iqtl];

    var1 = (1.0/effect_prior_var) + (work3/sigmasq);
    work1 = 0.0;
    for(jobs=0;jobs<ntrait;jobs++){
      work2 = 0.0;
      for(kqtl=0;kqtl<nqtl;kqtl++)
        if(kqtl!=iqtl) work2 = work2 +
                               effect[kqtl] * qtl_genotype[jobs][kqtl];
      work1 = work1 + qtl_genotype[jobs][iqtl] *
                      (y[jobs] - mu - work2);
    }
    mean1 = (effect_prior_mn/effect_prior_var + work1/sigmasq) / var1;
    /* effect_val[iqtl] = gennor(0.0, 1.0);
    effect_val[iqtl] = mean1 + effect_val[iqtl] / sqrt(var1); */
    work = gennor(0.0, 1.0);
    effect[iqtl] = mean1 + work / sqrt(var1);
  }
  /* return effect_val; */
}

/*************************************
    Function GET_NEW_SIGMASQ
*************************************/

double get_new_sigmasq(int ntrait, int nqtl, double *y,
                       int qtl_genotype[][MAXQTL], double mu,
                       double *effect, double prior_sigsq_a1,
                       double prior_sigsq_a2){
  int iqtl, jobs;
  double work1, work2, work3, sigmasq_val;

  sigmasq_val = gengam(1.0, prior_sigsq_a1);
  work1 = 0.0;
  for(jobs=0;jobs<ntrait;jobs++){
    work2 = 0.0;
    for(iqtl=0;iqtl<nqtl;iqtl++)
      work2 = work2+effect[iqtl]*qtl_genotype[jobs][iqtl];
    work3 = y[jobs]-mu-work2;
    work1 = work1 + work3 * work3;
  }
  sigmasq_val = (1.0/prior_sigsq_a2 + work1/2.0) / sigmasq_val;
  return sigmasq_val;
}
